<?php

/**
 * @file
 * Contains \Drupal\yamlform\Entity\YamlFormSubmissionListBuilder.
 */

namespace Drupal\yamlform;

use Drupal\Component\Render\FormattableMarkup;
use Drupal\Component\Serialization\Json;
use Drupal\Core\Database\Query\AlterableInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityListBuilder;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Link;
use Drupal\Core\Url;

/**
 * Provides a list controller for yamlform submission entity.
 *
 * @ingroup yamlform
 */
class YamlFormSubmissionListBuilder extends EntityListBuilder {

  /**
   * Submission state starred.
   */
  const STATE_STARRED = 'starred';

  /**
   * Submission state unstarred.
   */
  const STATE_UNSTARRED = 'unstarred';

  /**
   * The YAML form.
   *
   * @var \Drupal\yamlform\YamlFormInterface
   */
  protected $yamlform;

  /**
   * The entity that a YAML form is attached to. Currently only applies to nodes.
   *
   * @var \Drupal\Core\Entity\EntityInterface
   */
  protected $sourceEntity;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $account;

  /**
   * The table header columns.
   *
   * @var array
   */
  protected $header;

  /**
   * The YAML form elements.
   *
   * @var array
   */
  protected $elements = [];

  /**
   * Search keys.
   *
   * @var string
   */
  protected $keys;

  /**
   * Search state.
   *
   * @var string
   */
  protected $state;

  /**
   * The YAMl form element manager.
   *
   * @var \Drupal\yamlform\YamlFormElementManagerInterface
   */
  protected $elementManager;

  /**
   * {@inheritdoc}
   */
  public function __construct(EntityTypeInterface $entity_type, EntityStorageInterface $storage) {
    parent::__construct($entity_type, $storage);

    /** @var \Drupal\yamlform\YamlFormRequestInterface $yamlform_request */
    $yamlform_request = \Drupal::service('yamlform.request');

    $this->keys = \Drupal::request()->query->get('search');
    $this->state = \Drupal::request()->query->get('state');

    list($this->yamlform, $this->sourceEntity) = $yamlform_request->getYamlFormEntities();
    $base_route_name = $this->getBaseRouteName();

    $this->account = (\Drupal::routeMatch()->getRouteName() == "$base_route_name.yamlform.submissions") ? \Drupal::currentUser() : FALSE;

    if (\Drupal::routeMatch()->getRouteName() == "$base_route_name.yamlform.results_table") {
      $this->elements = $this->yamlform->getElementsFlattenedAndHasValue();
      // Use the default format when displaying each element.
      foreach ($this->elements as &$element) {
        unset($element['#format']);
      }
    }
    $this->elementManager = \Drupal::service('plugin.manager.yamlform.element');
  }

  /**
   * {@inheritdoc}
   */
  public function render() {
    if ($this->yamlform) {
      if ($this->account) {
        $build['#title'] = $this->t('Submissions to %yamlform for %user', [
          '%yamlform' => $this->yamlform->label(),
          '%user' => $this->account->getDisplayName(),
        ]);
      }
    }

    // Add the filter.
    $state_options = [
      '' => $this->t('All [@total]', ['@total' => $this->getTotal(NULL, NULL)]),
      'starred' => $this->t('Starred [@total]', ['@total' => $this->getTotal(NULL, self::STATE_STARRED)]),
      'unstarred' => $this->t('Unstarred [@total]', ['@total' => $this->getTotal(NULL, self::STATE_UNSTARRED)]),
    ];
    $build['filter_form'] = \Drupal::formBuilder()->getForm('\Drupal\yamlform\Form\YamlFormSubmissionFilterForm', $this->keys, $this->state, $state_options);

    // Display info.
    if ($total = $this->getTotal($this->state, $this->keys)) {
      $t_args = [
        '@total' => $total,
        '@results' => $this->formatPlural($total, $this->t('submission'), $this->t('submissions')),
      ];
      $build['info'] = [
        '#markup' => $this->t('@total @results', $t_args),
        '#prefix' => '<div>',
        '#suffix' => '</div>',
      ];
    }

    $build += parent::render();

    $build['table']['#attributes']['class'][] = 'yamlform-results';

    $build['#attached']['library'][] = 'yamlform/yamlform';

    return $build;
  }

  /**
   * {@inheritdoc}
   */
  public function buildHeader() {
    if (isset($this->header)) {
      return $this->header;
    }

    $view_any = ($this->yamlform && $this->yamlform->access('submission_view_any')) ? TRUE : FALSE;

    $header['sid'] = [
      'data' => $this->t('#'),
      'field' => 'sid',
      'specifier' => 'sid',
      'sort' => 'desc',
    ];

    $header['sticky'] = [
      'data' => $this->t('<span class="yamlform-icon yamlform-icon-sticky yamlform-icon-sticky-link"></span>'),
      'field' => 'sticky',
      'specifier' => 'sticky',
      'sort' => 'desc',
      'class' => ['yamlform-results-icon'],
    ];

    $header['notes'] = [
      'data' => $this->t('<span class="yamlform-icon yamlform-icon-notes yamlform-icon-notes-on"></span>'),
      'class' => ['yamlform-results-icon'],
    ];

    $header['created'] = [
      'data' => $this->t('Submitted'),
      'field' => 'created',
      'specifier' => 'created',
      'class' => [RESPONSIVE_PRIORITY_MEDIUM],
    ];

    if ($view_any && empty($this->sourceEntity)) {
      $header['entity'] = [
        'data' => $this->t('Submitted to'),
      ];
    }

    if (empty($this->account)) {
      $header['uid'] = [
        'data' => $this->t('User'),
        'field' => 'uid',
        'specifier' => 'uid',
        'class' => [RESPONSIVE_PRIORITY_MEDIUM],
      ];
    }

    if ($view_any && $this->moduleHandler()->moduleExists('language')) {
      $header['langcode'] = [
        'data' => $this->t('Language'),
        'field' => 'langcode',
        'specifier' => 'langcode',
        'class' => [RESPONSIVE_PRIORITY_LOW],
      ];
    }

    $header['remote_addr'] = [
      'data' => $this->t('IP address'),
      'field' => 'remote_addr',
      'specifier' => 'remote_addr',
      'class' => [RESPONSIVE_PRIORITY_LOW],
    ];

    if (empty($this->yamlform) && empty($this->sourceEntity)) {
      $header['yamlform'] = [
        'data' => $this->t('Form'),
        'class' => [RESPONSIVE_PRIORITY_LOW],
      ];
    }

    if ($this->elements) {
      foreach ($this->elements as $key => $element) {
        $header['element_' . $key] = $element['#title'] ?: $key;
      }
    }

    // Cache header in protected variable.
    $this->header = $header + parent::buildHeader();
    return $this->header;
  }

  /**
   * {@inheritdoc}
   */
  public function buildRow(EntityInterface $entity) {
    /* @var $entity \Drupal\yamlform\YamlFormSubmissionInterface */
    $view_any = ($this->yamlform && $this->yamlform->access('submission_view_any')) ? TRUE : FALSE;

    $route_name = $this->getBaseRouteName() . '.yamlform_submission.canonical';
    $route_parameters = $this->getRouteParameters($entity);
    $link_text = $entity->id() . ($entity->isDraft() ? ' (' . $this->t('draft') . ')' : '');
    $row['sid'] = Link::createFromRoute($link_text, $route_name, $route_parameters);

    $row['sticky'] = $this->buildStickyLink($entity);

    $row['notes'] = $this->buildNotesLink($entity);

    $row['created'] = \Drupal::service('date.formatter')->format($entity->created->value);

    if ($view_any && !$this->sourceEntity) {
      $row['entity'] = ($source_entity = $entity->getSourceEntity()) ? $source_entity->toLink() : '';
    }

    if (empty($this->account)) {
      $row['user'] = $entity->getOwner()->getAccountName() ?: t('Anonymous');
    }

    if ($view_any && $this->moduleHandler()->moduleExists('language')) {
      $row['langcode'] = \Drupal::languageManager()->getLanguage($entity->langcode->value)->getName();
    }

    $row['remote_addr'] = $entity->getRemoteAddr();

    if (empty($this->yamlform) && empty($this->sourceEntity)) {
      $row['yamlform'] = $entity->getYamlForm()->toLink();
    }

    if ($this->elements) {
      $data = $entity->getData();
      foreach ($this->elements as $key => $element) {
        $options = [];
        $html = $this->elementManager->invokeMethod('formatHtml', $element, $data[$key], $options);
        if (is_array($html)) {
          $row['element_' . $key] = ['data' => $html];
        }
        else {
          $row['element_' . $key] = $html;
        }
      }
    }

    return $row + parent::buildRow($entity);
  }

  /**
   * {@inheritdoc}
   */
  public function getDefaultOperations(EntityInterface $entity) {
    $base_route_name = $this->getBaseRouteName();
    $route_parameters = $this->getRouteParameters($entity);
    $route_options = ['query' => \Drupal::destination()->getAsArray()];

    $operations = [];

    if ($entity->access('update')) {
      $operations['edit'] = [
        'title' => $this->t('Edit'),
        'weight' => 10,
        'url' => Url::fromRoute("$base_route_name.yamlform_submission.edit_form", $route_parameters, $route_options),
      ];
    }

    if ($entity->access('view')) {
      $operations['view'] = [
        'title' => $this->t('View'),
        'weight' => 20,
        'url' => Url::fromRoute("$base_route_name.yamlform_submission.canonical", $route_parameters),
      ];
    }

    if ($entity->access('update')) {
      $operations['notes'] = [
        'title' => $this->t('Notes'),
        'weight' => 21,
        'url' => Url::fromRoute("$base_route_name.yamlform_submission.notes_form", $route_parameters, $route_options),
      ];
      $operations['resend'] = [
        'title' => $this->t('Resend'),
        'weight' => 22,
        'url' => Url::fromRoute("$base_route_name.yamlform_submission.resend_form", $route_parameters, $route_options),
      ];
    }

    if ($entity->access('delete')) {
      $operations['delete'] = [
        'title' => $this->t('Delete'),
        'weight' => 100,
        'url' => Url::fromRoute("$base_route_name.yamlform_submission.delete_form", $route_parameters, $route_options),
      ];
    }

    return $operations;
  }

  /**
   * {@inheritdoc}
   */
  protected function getEntityIds() {
    $header = $this->buildHeader();
    return $this->getQuery($this->keys, $this->state)
      ->pager($this->limit)
      ->tableSort($header)
      ->execute();
  }

  /**
   * Get base route name for the YAML form or YAML form source entity.
   *
   * @return string
   *   The base route name for YAML form or YAML form source entity.
   */
  protected function getBaseRouteName() {
    if ($this->sourceEntity) {
      return 'entity.' . $this->sourceEntity->getEntityTypeId();
    }
    else {
      return 'entity';
    }
  }

  /**
   * Get route parameters for the YAML form or YAML form source entity.
   *
   * @param \Drupal\yamlform\YamlFormSubmissionInterface $yamlform_submission
   *   A YAML form submission.
   *
   * @return array
   *   Route parameters for the YAML form or YAML form source entity.
   */
  protected function getRouteParameters(YamlFormSubmissionInterface $yamlform_submission) {
    $router_parameters = ['yamlform_submission' => $yamlform_submission->id()];
    if ($this->sourceEntity) {
      $router_parameters[$this->sourceEntity->getEntityTypeId()] = $this->sourceEntity->id();
    }
    return $router_parameters;
  }

  /**
   * Get the total number of submissions.
   *
   * @param string $keys
   *   (optional) Search key.
   * @param string $state
   *   (optional) Submission state. Can be 'starred' or 'unstarred'.
   *
   * @return int
   *   The total number of submissions.
   */
  protected function getTotal($keys = '', $state = '') {
    return $this->getQuery($keys, $state)
      ->count()
      ->execute();
  }

  /**
   * Get the base entity query filtered by YAML form and search.
   *
   * @param string $keys
   *   (optional) Search key.
   * @param string $state
   *   (optional) Submission state. Can be 'starred' or 'unstarred'.
   *
   * @return \Drupal\Core\Entity\Query\QueryInterface
   *   An entity query.
   */
  protected function getQuery($keys = '', $state = '') {
    $query = $this->getStorage()->getQuery();
    $this->addQueryConditions($query);

    // Filter by key(word).
    if ($keys) {
      $sub_query = db_select('yamlform_submission_data', 'sd')
        ->fields('sd', ['sid'])
        ->condition('value', '%' . $keys . '%', 'LIKE');
      $this->addQueryConditions($sub_query);

      $or = $query->orConditionGroup()
        ->condition('sid', $sub_query, 'IN')
        ->condition('notes', '%' . $keys . '%', 'LIKE');

      $query->condition($or);
    }

    // Filter by (submission) state.
    if ($state == self::STATE_STARRED || $state == self::STATE_UNSTARRED) {
      $query->condition('sticky', ($state == self::STATE_STARRED) ? 1 : 0);
    }

    return $query;
  }

  /**
   * Add YAML form, account, and source entity conditions to a query.
   *
   * @param \Drupal\Core\Database\Query\AlterableInterface $query
   *   The query to execute.
   */
  protected function addQueryConditions(AlterableInterface $query) {
    // Limit submission to the current YAML form.
    if ($this->yamlform) {
      $query->condition('yamlform_id', $this->yamlform->id());
    }

    // Limit submission to the current user.
    if ($this->account) {
      $query->condition('uid', $this->account->id());
    }

    // Filter entity type and id. (Currently only applies to yamlform_node.module)
    if ($this->sourceEntity) {
      $query->condition('entity_type', $this->sourceEntity->getEntityTypeId());
      $query->condition('entity_id', $this->sourceEntity->id());
    }
  }

  /**
   * Build link to a submission notes dialog.
   *
   * @param \Drupal\yamlform\YamlFormSubmissionInterface $yamlform_submission
   *   A YAML form submission.
   *
   * @return array
   *   A renderable link.
   */
  protected function buildNotesLink(YamlFormSubmissionInterface $yamlform_submission) {
    $route_name = $this->getBaseRouteName() . '.yamlform_submission.notes_form';
    $route_parameters = $this->getRouteParameters($yamlform_submission);
    $route_options = ['query' => \Drupal::destination()->getAsArray()];
    $state = $yamlform_submission->get('notes')->value ? 'on' : 'off';
    $dialog_options = Json::encode([
      'width' => 400,
    ]);

    return [
      'data' => [
        '#type' => 'link',
        '#title' => new FormattableMarkup('<span class="yamlform-icon yamlform-icon-notes yamlform-icon-notes-@state"></span>', ['@state' => $state]),
        '#url' => Url::fromRoute($route_name, $route_parameters, $route_options),
        '#attributes' => [
          'class' => ['use-ajax'],
          'data-dialog-type' => 'modal',
          'data-dialog-options' => $dialog_options,
        ],
      ],
      'class' => ['yamlform-results-icon'],
    ];
  }

  /**
   * Build link to a submission sticky.
   *
   * @param \Drupal\yamlform\YamlFormSubmissionInterface $yamlform_submission
   *   A YAML form submission.
   *
   * @return array
   *   A renderable link.
   */
  protected function buildStickyLink(YamlFormSubmissionInterface $yamlform_submission) {
    $route_name = 'entity.yamlform_submission.sticky_toggle';
    $route_parameters = ['yamlform_submission' => $yamlform_submission->id()];
    $state = $yamlform_submission->isSticky() ? 'on' : 'off';
    return [
      'data' => [
        '#type' => 'link',
        '#title' => new FormattableMarkup('<span class="yamlform-icon yamlform-icon-sticky yamlform-icon-sticky-@state"></span>', ['@state' => $state]),
        '#url' => Url::fromRoute($route_name, $route_parameters),
        '#attributes' => [
          'id' => 'yamlform-submission-' . $yamlform_submission->id() . '-sticky',
          'class' => ['use-ajax'],
        ],
      ],
      'class' => ['yamlform-results-icon'],
    ];
  }

}
